# coding: utf-8

"""
    Enterprise Mission Assurance Support Service (eMASS)

    The eMASS Representational State Transfer (REST) Application Programming Interface (API) enables users to perform assessments and complete actions associated with system records.  The eMASS API provides an interface for application to communicate eMASS Services. For information on how to register and use the eMASS API reference the [eMASS API Getting Started](eMASSGettingStarted.md).  Additional information about eMASS can be obtain by contacting the National Industrial Security Program (NISP). Points of Contact are: 

    The version of the OpenAPI document: v3.22
    Contact: disa.global.servicedesk.mbx.ma-ticket-request@mail.mil
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class ControlsRequiredFields(BaseModel):
    """
    ControlsRequiredFields
    """ # noqa: E501
    acronym: Optional[StrictStr] = Field(default=None, description="[Required] Acronym of the system record.")
    responsible_entities: Optional[StrictStr] = Field(default=None, description="[Required] Include written description of Responsible Entities that are responsible for the Security Control. Character Limit 2,000.", alias="responsibleEntities")
    control_designation: Optional[StrictStr] = Field(default=None, description="[Required] Control designations", alias="controlDesignation")
    estimated_completion_date: Optional[StrictInt] = Field(default=None, description="[Required] Field is required for Implementation Plan. Unix time format.", alias="estimatedCompletionDate")
    implementation_narrative: Optional[StrictStr] = Field(default=None, description="[Required] Includes security control comments. Character Limit 2,000.", alias="implementationNarrative")
    __properties: ClassVar[List[str]] = ["acronym", "responsibleEntities", "controlDesignation", "estimatedCompletionDate", "implementationNarrative"]

    @field_validator('control_designation')
    def control_designation_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Common', 'System-Specific', 'Hybrid']):
            raise ValueError("must be one of enum values ('Common', 'System-Specific', 'Hybrid')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ControlsRequiredFields from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ControlsRequiredFields from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "acronym": obj.get("acronym"),
            "responsibleEntities": obj.get("responsibleEntities"),
            "controlDesignation": obj.get("controlDesignation"),
            "estimatedCompletionDate": obj.get("estimatedCompletionDate"),
            "implementationNarrative": obj.get("implementationNarrative")
        })
        return _obj


