# coding: utf-8

"""
    Enterprise Mission Assurance Support Service (eMASS)

    The eMASS Representational State Transfer (REST) Application Programming Interface (API) enables users to perform assessments and complete actions associated with system records.  The eMASS API provides an interface for application to communicate eMASS Services. For information on how to register and use the eMASS API reference the [eMASS API Getting Started](eMASSGettingStarted.md).  Additional information about eMASS can be obtain by contacting the National Industrial Security Program (NISP). Points of Contact are: 

    The version of the OpenAPI document: v3.22
    Contact: disa.global.servicedesk.mbx.ma-ticket-request@mail.mil
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class PacGet(BaseModel):
    """
    PacGet
    """ # noqa: E501
    workflow: Optional[StrictStr] = Field(default=None, description="[Required] Values include the following:(Assess and Authorize, Assess Only, Security Plan Approval)")
    name: Optional[StrictStr] = Field(default=None, description="[Required] Package name. 100 Characters.")
    current_stage_name: Optional[StrictStr] = Field(default=None, description="[Read-Only] Name of the current stage in the active workflow.", alias="currentStageName")
    current_stage: Optional[StrictInt] = Field(default=None, description="[Read-Only] Number of the current stage in the active workflow.", alias="currentStage")
    total_stages: Optional[StrictInt] = Field(default=None, description="[Read-Only] Total number of stages in the active workflow.", alias="totalStages")
    days_at_current_stage: Optional[StrictInt] = Field(default=None, description="[Read-Only] Indicates the number of days at current workflow stage.", alias="daysAtCurrentStage")
    comments: Optional[StrictStr] = Field(default=None, description="[Required]] Comments submitted upon initiation of the indicated workflow. Character Limit = 4,000.")
    __properties: ClassVar[List[str]] = ["workflow", "name", "currentStageName", "currentStage", "totalStages", "daysAtCurrentStage", "comments"]

    @field_validator('workflow')
    def workflow_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Assess and Authorize', 'Assess Only', 'Security Plan Approval']):
            raise ValueError("must be one of enum values ('Assess and Authorize', 'Assess Only', 'Security Plan Approval')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PacGet from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if current_stage_name (nullable) is None
        # and model_fields_set contains the field
        if self.current_stage_name is None and "current_stage_name" in self.model_fields_set:
            _dict['currentStageName'] = None

        # set to None if current_stage (nullable) is None
        # and model_fields_set contains the field
        if self.current_stage is None and "current_stage" in self.model_fields_set:
            _dict['currentStage'] = None

        # set to None if total_stages (nullable) is None
        # and model_fields_set contains the field
        if self.total_stages is None and "total_stages" in self.model_fields_set:
            _dict['totalStages'] = None

        # set to None if days_at_current_stage (nullable) is None
        # and model_fields_set contains the field
        if self.days_at_current_stage is None and "days_at_current_stage" in self.model_fields_set:
            _dict['daysAtCurrentStage'] = None

        # set to None if comments (nullable) is None
        # and model_fields_set contains the field
        if self.comments is None and "comments" in self.model_fields_set:
            _dict['comments'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PacGet from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "workflow": obj.get("workflow"),
            "name": obj.get("name"),
            "currentStageName": obj.get("currentStageName"),
            "currentStage": obj.get("currentStage"),
            "totalStages": obj.get("totalStages"),
            "daysAtCurrentStage": obj.get("daysAtCurrentStage"),
            "comments": obj.get("comments")
        })
        return _obj


