# coding: utf-8

"""
    Enterprise Mission Assurance Support Service (eMASS)

    The eMASS Representational State Transfer (REST) Application Programming Interface (API) enables users to perform assessments and complete actions associated with system records.  The eMASS API provides an interface for application to communicate eMASS Services. For information on how to register and use the eMASS API reference the [eMASS API Getting Started](eMASSGettingStarted.md).  Additional information about eMASS can be obtain by contacting the National Industrial Security Program (NISP). Points of Contact are: 

    The version of the OpenAPI document: v3.22
    Contact: disa.global.servicedesk.mbx.ma-ticket-request@mail.mil
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from emass_client.models.connectivity_ccsd import ConnectivityCcsd
from typing import Optional, Set
from typing_extensions import Self

class Systems(BaseModel):
    """
    Systems
    """ # noqa: E501
    registration_completion_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Date the system was registered into eMASS.", alias="registrationCompletionDate")
    system_life_cycle_acquisition_phase: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identifies the current System Acquisition Phase for programs of record.", alias="systemLifeCycleAcquisitionPhase")
    special_type: Optional[StrictStr] = Field(default=None, description="[Read-Only] Lists applicable tracking indicator(s).", alias="specialType")
    special_type_description: Optional[StrictStr] = Field(default=None, description="[Read-Only] Provides a brief reason for any tracking indicator(s) selected.", alias="specialTypeDescription")
    mission_portfolio: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identifies the appropriate portfolio or capability area. Navy only.", alias="missionPortfolio")
    is_nnpi: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicates whether Naval Nuclear Propulsion Information (NNPI) is stored, disseminated, or processed through this system. Navy only.", alias="isNNPI")
    is_rbc: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicates whether the system is pursuing an RBC authorization. Navy only.", alias="isRBC")
    is_waiver: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicates if the system has a waiver from OPNAV N2N6G (DDCIO(N)) to proceed with a DIACAP accreditation. Navy and DIACAP only.", alias="isWaiver")
    program_office: Optional[StrictStr] = Field(default=None, description="[Read-Only] The system record's Program Office. Navy only.", alias="programOffice")
    vram_id: Optional[StrictStr] = Field(default=None, description="[Read-Only] Vulnerability Remediation Asset Manager (VRAM) identification number. \"N/A\" indicates the system record is not currently registered in VRAM.  Navy only.", alias="vramId")
    system_id: Optional[StrictInt] = Field(default=None, description="[Read-only] Unique system record identifier.", alias="systemId")
    policy: Optional[StrictStr] = Field(default=None, description="[Read-only] RMF/DIACAP Policy identifier for the system record.")
    registration_type: Optional[StrictStr] = Field(default=None, description="[Read-Only] Registration type of the system record. Values include (assessAndAuthorize, assessOnly, guest, regular, functional, cloudServiceProvider.)", alias="registrationType")
    name: Optional[StrictStr] = Field(default=None, description="[Read-only] Name of the system record.")
    acronym: Optional[StrictStr] = Field(default=None, description="[Read-only] Acronym of the system record.")
    description: Optional[StrictStr] = Field(default=None, description="[Read-only] Description of the system record.")
    instance: Optional[StrictStr] = Field(default=None, description="[Read-Only] Name of the top-level component that owns the system.")
    owning_organization: Optional[StrictStr] = Field(default=None, description="[Read-Only] Owning organization of the system record. Values match the eMASS instance Organizational Hierarchy.", alias="owningOrganization")
    secondary_organization: Optional[StrictStr] = Field(default=None, description="[Read-only] Secondary organization that owns the system record (i.e. Sub-Organization-level.", alias="secondaryOrganization")
    version_release_no: Optional[StrictStr] = Field(default=None, description="[Read-only] Version/Release Number of system record.", alias="versionReleaseNo")
    system_type: Optional[StrictStr] = Field(default=None, description="[Read-only] Type of the system record. RMF values include the following options (IS Major Application, IS Enclave, Platform IT System). DIACAP values include the following options (Platform IT Interconnection, AIS Application, Outsourced IT-Based Process (DoD-controlled), Enclave, Outsourced IT-Based Process (service provider shared))", alias="systemType")
    is_nss: Optional[StrictBool] = Field(default=None, description="[Read-only] Is the system record a National Security System?", alias="isNSS")
    is_public_facing: Optional[StrictBool] = Field(default=None, description="[Read-only] Does the system record have a public facing component/presence.", alias="isPublicFacing")
    coams_id: Optional[StrictInt] = Field(default=None, description="[Read-only] Corresponding Cyber Operational Attributes Management System (COAMS) identifier for the system record.", alias="coamsId")
    is_type_authorization: Optional[StrictBool] = Field(default=None, description="[Read-only] Identifies if system is a Type Authorization.", alias="isTypeAuthorization")
    ditpr_id: Optional[StrictStr] = Field(default=None, description="[Read-only] DITPR ID of the system record.", alias="ditprId")
    apms_id: Optional[StrictStr] = Field(default=None, description="[Read-Only] Same field as ditprId but displays as apmsId for Army only.", alias="apmsId")
    vasi_id: Optional[StrictStr] = Field(default=None, description="[Read-Only] Same field as ditprId but displays as vasiId for VA only.", alias="vasiId")
    authorization_status: Optional[StrictStr] = Field(default=None, description="[Read-only] Authorization Status of the system record.", alias="authorizationStatus")
    authorization_date: Optional[StrictInt] = Field(default=None, description="[Read-only] Authorization Date of the system record.", alias="authorizationDate")
    authorization_termination_date: Optional[StrictInt] = Field(default=None, description="[Read-only] Authorization Termination Date of the system record.", alias="authorizationTerminationDate")
    authorization_length: Optional[Annotated[int, Field(le=1825, strict=True, ge=28)]] = Field(default=None, description="[Read-only] Length of system's Authorization. Calculated based off of Authorization Date & Authorization Termination Date.", alias="authorizationLength")
    security_controls_assessor_executive_summary: Optional[StrictStr] = Field(default=None, description="[Read-Only] Document an assessment of overall cybersecurity risk (Very Low, Low, Moderate, High, Very High) and identify the key drivers for assessment.", alias="securityControlsAssessorExecutiveSummary")
    risk_review_executive_summary: Optional[StrictStr] = Field(default=None, description="[Read-Only] Document an assessment of overall cybersecurity risk (Very Low, Low, Moderate, High, Very High) and identify the key drivers for assessment. VA only", alias="riskReviewExecutiveSummary")
    terms_for_auth: Optional[StrictStr] = Field(default=None, description="[Read-only] Terms/Conditions for receiving and maintaining the system's Authorization. Assigned by the Authorizing Official.", alias="termsForAuth")
    security_plan_approval_status: Optional[StrictStr] = Field(default=None, description="[Read-only] Status of the approval of the system's RMF Security Plan. Values include the following options (Approved, Denied, Not Yet Approved).", alias="securityPlanApprovalStatus")
    security_plan_approval_date: Optional[StrictInt] = Field(default=None, description="[Read-only] Approval date of the system's RMF Security Plan.", alias="securityPlanApprovalDate")
    mission_criticality: Optional[StrictStr] = Field(default=None, description="[Read-only] Mission Criticality of the system record.", alias="missionCriticality")
    geographical_association: Optional[StrictStr] = Field(default=None, description="[Read-only] Geographical Association of the system record.", alias="geographicalAssociation")
    system_ownership_controlled: Optional[StrictStr] = Field(default=None, description="[Read-only] Ownership of the system record.", alias="systemOwnershipControlled")
    governing_mission_area: Optional[StrictStr] = Field(default=None, description="[Read-only] Governing Mission Area of the system record.", alias="governingMissionArea")
    primary_functional_area: Optional[StrictStr] = Field(default=None, description="[Read-only] Primary functional area of the system record.", alias="primaryFunctionalArea")
    secondary_functional_area: Optional[StrictStr] = Field(default=None, description="[Read-only] Secondary functional area of the system record.", alias="secondaryFunctionalArea")
    primary_control_set: Optional[StrictStr] = Field(default=None, description="[Read-only] Primary Control Set of the system record. RMF values include the following options (NIST SP 800-53 Revision 4), DIACAP values include the following options (DoDI 8500.2)", alias="primaryControlSet")
    confidentiality: Optional[StrictStr] = Field(default=None, description="[Read-only] Confidentiality of the system record. RMF values include the following options (High, Moderate, Low)")
    integrity: Optional[StrictStr] = Field(default=None, description="[Read-only] Integrity of the system record. RMF values include the following options (High, Moderate, Low)")
    availability: Optional[StrictStr] = Field(default=None, description="[Read-only] Availability of the system record. RMF values include the following options (High, Moderate, Low)")
    applied_overlays: Optional[StrictStr] = Field(default=None, description="[Read-only] Overlays applied to the system record. Multiple values are separated by semi-colon ';'.", alias="appliedOverlays")
    applied_stigs: Optional[StrictStr] = Field(default=None, description="[Read-only] The STIG's applied to the system record.", alias="appliedStigs")
    rmf_activity: Optional[StrictStr] = Field(default=None, description="[Read-only] RMF Activity of the system record.", alias="rmfActivity")
    cross_domain_ticket: Optional[StrictStr] = Field(default=None, description="[Read-only] Cross Domain Tickets of the system record.", alias="crossDomainTicket")
    ditpr_don_id: Optional[StrictStr] = Field(default=None, description="[Read-Only] DITPR-DON identifier of the system record.", alias="ditprDonId")
    mac: Optional[StrictStr] = Field(default=None, description="[Read-Only] MAC level of the system record.")
    dod_confidentiality: Optional[StrictStr] = Field(default=None, description="[Read-Only] DoD Confidentiality level of the system record.", alias="dodConfidentiality")
    contingency_plan_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is there a Contingency Plan in place for this system that addresses disruptions in operations?", alias="contingencyPlanRequired")
    contingency_plan_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's Contingency Plan artifact.", alias="contingencyPlanArtifact")
    contingency_plan_tested: Optional[StrictBool] = Field(default=None, description="[Read-only] Has the system record's Contingency Plan been tested?", alias="contingencyPlanTested")
    contingency_plan_test_date: Optional[StrictInt] = Field(default=None, description="[Read-only] Date the system record's Contingency Plan was tested.", alias="contingencyPlanTestDate")
    security_review_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is the system required to complete a Security Review?", alias="securityReviewRequired")
    security_review_completed: Optional[StrictBool] = Field(default=None, description="[Read-Only] Has a Security Review been completed for this system?", alias="securityReviewCompleted")
    security_review_completion_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Date of the system's latest security review or annual assessment.", alias="securityReviewCompletionDate")
    next_security_review_due_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Date when the system's next security review or annual assessment is due by.", alias="nextSecurityReviewDueDate")
    has_open_poam_item: Optional[StrictBool] = Field(default=None, description="[Read-Only] Does the system record have an Ongoing or Risk Accepted POA&M Item?", alias="hasOpenPoamItem")
    has_open_poam_item90to120_past_scheduled_completion_date: Optional[StrictBool] = Field(default=None, description="[Read-Only] Does the system record have an Ongoing or Risk Accepted POA&M Item 90 to 120 days past its Scheduled Completion Date?", alias="hasOpenPoamItem90to120PastScheduledCompletionDate")
    has_open_poam_item120_plus_past_scheudled_completion_date: Optional[StrictBool] = Field(default=None, description="[Read-Only] Does the system record have an Ongoing or Risk Accepted POA&M Item 120 days past its Scheduled Completion Date?", alias="hasOpenPoamItem120PlusPastScheudledCompletionDate")
    impact: Optional[StrictStr] = Field(default=None, description="[Optional] Values include the following options (Very Low, Low, Moderate,High,Very High)")
    has_cui: Optional[StrictBool] = Field(default=None, description="[Read-only] Does the system record contain and/or process Controlled Unclassified information?", alias="hasCUI")
    has_pii: Optional[StrictBool] = Field(default=None, description="[Read-only] Does the system record contain and/or process Personally Identifiable Information?", alias="hasPII")
    has_phi: Optional[StrictBool] = Field(default=None, description="[Read-only] Does the system record contain and/or process Personal Health Information?", alias="hasPHI")
    ppsm_registration_required: Optional[StrictStr] = Field(default=None, description="[Read-Only] Determine if a PPSM registration is required.", alias="ppsmRegistrationRequired")
    ppsm_registry_number: Optional[StrictStr] = Field(default=None, description="[Read-only] Unique identifier for the DoD's Ports, Protocols, and Services Management Registry system.", alias="ppsmRegistryNumber")
    ppsm_registration_exemption_justification: Optional[StrictStr] = Field(default=None, description="[Read-Only] Clarify why a PPSM registraiton is not necessary.", alias="ppsmRegistrationExemptionJustification")
    interconnected_information_system_and_identifiers: Optional[StrictStr] = Field(default=None, description="[Read-only] Identify the interconnected information systems and corresponding identifiers within control CA-3.", alias="interconnectedInformationSystemAndIdentifiers")
    privacy_impact_assessment_required: Optional[StrictBool] = Field(default=None, description="[Read-only] Does the system require a Privacy Impact Assessment?", alias="privacyImpactAssessmentRequired")
    privacy_impact_assessment_status: Optional[StrictStr] = Field(default=None, description="[Read-only] Status of the PIA, availability values include the following options (Not Started, In Progress, Completed)", alias="privacyImpactAssessmentStatus")
    privacy_impact_assessment_date: Optional[StrictInt] = Field(default=None, description="[Read-only] Date in which the system's PIA took place.", alias="privacyImpactAssessmentDate")
    privacy_impact_assessment_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's PIA artifact.", alias="privacyImpactAssessmentArtifact")
    user_defined_field1: Optional[StrictStr] = Field(default=None, description="[Read-only] User-defined field to augment Ad Hoc Reporting.", alias="userDefinedField1")
    user_defined_field2: Optional[StrictStr] = Field(default=None, description="[Read-only] User-defined field to augment Ad Hoc Reporting.", alias="userDefinedField2")
    user_defined_field3: Optional[StrictStr] = Field(default=None, description="[Read-only] User-defined field to augment Ad Hoc Reporting.", alias="userDefinedField3")
    user_defined_field4: Optional[StrictStr] = Field(default=None, description="[Read-only] User-defined field to augment Ad Hoc Reporting.", alias="userDefinedField4")
    user_defined_field5: Optional[StrictStr] = Field(default=None, description="[Read-only] User-defined field to augment Ad Hoc Reporting.", alias="userDefinedField5")
    current_rmf_lifecycle_step: Optional[StrictStr] = Field(default=None, description="[Read-only] Displays the system's current step within the RMF Lifecycle.", alias="currentRmfLifecycleStep")
    other_information: Optional[StrictStr] = Field(default=None, description="[Read-only] Include any additional information required by the organization.", alias="otherInformation")
    reports_for_scorecard: Optional[StrictBool] = Field(default=None, description="[Read-only] Indicates if the system reports to the DoD Cyber Hygiene Scorecard.", alias="reportsForScorecard")
    highest_system_data_classification: Optional[StrictStr] = Field(default=None, description="[Read-Only] The overall classification level of information that the System is approved to collect, process, store, and/or distribute.", alias="highestSystemDataClassification")
    overall_classification: Optional[StrictStr] = Field(default=None, description="[Read-Only] Same field as highestSystemDataClassification, but displays as overallClassification for NISP only.", alias="overallClassification")
    is_hva: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicates if the system contains High Value Assets. Does not display if value is null", alias="isHVA")
    is_financial_management: Optional[StrictBool] = Field(default=None, description="[Read-Only] Per OMB Circular A-127, a financial management system includes the core financial systems and the financial portions of mixed systems necessary to support financial management, including automated and manual processes, procedures, and controls, data, hardware, software, and support personnel dedicated to the operation and maintenance of system functions. The following are examples of financial management systems: core financial systems, procurement systems, loan systems, grants systems, payroll systems, budget formulation systems, billing systems, and travel systems. ", alias="isFinancialManagement")
    is_reciprocity: Optional[StrictBool] = Field(default=None, description="[Read-Only] A reciprocity system is any information system that is part of a mutual agreement among participating organizations to accept each other's security assessments in order to reuse information system resources and/or to accept each other's assessed security posture in order to share information. ", alias="isReciprocity")
    reciprocity_exemption: Optional[StrictStr] = Field(default=None, description="[Read-Only] The following justifications are acceptable for exemption from reciprocity: (a) the existence of the system is classified (not the data, but the existence of the system) or (b) the system's authorization to operate is in the process of being pulled (e.g. DATO, Decommission). ", alias="reciprocityExemption")
    cloud_computing: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is this a cloud-based IS?", alias="cloudComputing")
    cloud_type: Optional[StrictStr] = Field(default=None, description="[Read-Only] Values include the following: (Hybrid, Private, Public)", alias="cloudType")
    authorization_to_connect_status: Optional[StrictStr] = Field(default=None, description="[Read-Only] The Authority to Connect decision. Values include the following: (Authority to Connect (ATC), Denial of Authority to Connect (DATC), Not Yet Connected, Decommissioned) ", alias="authorizationToConnectStatus")
    is_saa_s: Optional[StrictBool] = Field(default=None, description="[Read-Only] Software as a Service (SaaS) cloud service model.", alias="isSaaS")
    is_paa_s: Optional[StrictBool] = Field(default=None, description="[Read-Only] Platform as a Service (PaaS) cloud service model.", alias="isPaaS")
    is_iaa_s: Optional[StrictBool] = Field(default=None, description="[Read-Only] Infrastructure as a Service (IaaS) cloud service model.", alias="isIaaS")
    other_service_models: Optional[StrictStr] = Field(default=None, description="[Read-Only] Free text field to include other cloud service models.", alias="otherServiceModels")
    need_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Indicates the date by which the System needs to be deployed to a production environment.", alias="needDate")
    overall_risk_score: Optional[StrictStr] = Field(default=None, description="[Read-Only] The overall risk score of the system", alias="overallRiskScore")
    is_hrr: Optional[StrictBool] = Field(default=None, description="[Read-Only] Identifies whether a System has been designated as High Risk Review. USCG and Navy only.", alias="isHRR")
    connectivity_authorization_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The Connectivity Authorization Date.", alias="connectivityAuthorizationDate")
    connectivity_authorization_termination_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The Connectivity Authorization Termination Date.", alias="connectivityAuthorizationTerminationDate")
    system_development_life_cycle: Optional[StrictStr] = Field(default=None, description="[Read-Only] Indicate the date by which the System needs to be deployed to a production environment. VA only.", alias="systemDevelopmentLifeCycle")
    is_fisma_reportable: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is this IS reportable per Federal Information Security Management Act (FISMA) established requirements? VA only", alias="isFISMAReportable")
    group_tagging: Optional[StrictStr] = Field(default=None, description="[Read-Only] System Tags for enterprise level, to include CIO and CISO, tracking efforts. VA only.", alias="groupTagging")
    group_tag_descriptions: Optional[StrictStr] = Field(default=None, description="[Read-Only] System Tag explanation(s) for enterprise level, to include CIO and CISO, tracking efforts. VA only.", alias="groupTagDescriptions")
    dadms_id: Optional[StrictStr] = Field(default=None, description="[Read-Only] The system's DADMS ID. USMC only.", alias="dadmsId")
    dadms_expiration_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Date the system expires in DADMS. USMC only.", alias="dadmsExpirationDate")
    enclave_connectivity: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identify the type of connectivity for the network/enclave, e.g., DISA circuit (NIPR, SIPR) or HPCMP circuit (DREN, SDREN, Outreach). Navy only.", alias="enclaveConnectivity")
    environment_type: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identify the primary computing environment for where the information system is deployed. Navy only.", alias="environmentType")
    navy_common_control_provider: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicate whether the information system provides inheritable controls. Navy only", alias="navyCommonControlProvider")
    navy_cloud_broker: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identify the broker responsible for the delivery of commercial cloud services and capabilities. Refer to Navy Commercial Cloud Brokerage Policy. Navy Only", alias="navyCloudBroker")
    cloud_broker_emass_id: Optional[StrictInt] = Field(default=None, description="[Read-Only] The eMASS ID of the identified cloud broker. Navy only.", alias="cloudBrokerEmassId")
    cloud_broker_provisional_authorization_atd: Optional[StrictInt] = Field(default=None, description="[Read-Only] The provisional authorization termination date of the identified cloud broker. Navy only", alias="cloudBrokerProvisionalAuthorizationAtd")
    navy_joint_authorization: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicate whether this is a joint authorization being issued by two or more Authorizing Officials. Navy only", alias="navyJointAuthorization")
    nmci_ngen_clins: Optional[StrictStr] = Field(default=None, description="[Read-Only] Provide all NMCI CLINs associated to the system/services within the authorization boundary. Navy only", alias="nmciNgenClins")
    enterprise_locations: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identify the Navy enterprise network where the information system is deployed. Navy only", alias="enterpriseLocations")
    whitelist_id: Optional[StrictStr] = Field(default=None, description="[Read-Only] Systems that have public-facing components/presences are typically required to be documented and registered as part of Organzationally-approved whitelisting processes.", alias="whitelistId")
    whitelist_inventory: Optional[StrictStr] = Field(default=None, description="[Read-Only] Provide/upload the documentation that identifies or describes the components or aspects of the System that are public-facing (whitelisted).", alias="whitelistInventory")
    acquisition_category: Optional[StrictStr] = Field(default=None, description="[Read-Only] Values include the following options (I,II,III)", alias="acquisitionCategory")
    software_category: Optional[StrictStr] = Field(default=None, description="[Read-Only] Lists the software category.", alias="softwareCategory")
    cybersecurity_service_provider: Optional[StrictStr] = Field(default=None, description="[Read-Only] Name of the system's Cybersecurity Service Provider.", alias="cybersecurityServiceProvider")
    cybersecurity_service_provider_exception_justification: Optional[StrictStr] = Field(default=None, description="[Read-Only] If Not Applicable, provide the exception justification.", alias="cybersecurityServiceProviderExceptionJustification")
    maximum_tolerable_downtime: Optional[StrictStr] = Field(default=None, description="[Read-Only] MTD represents the total amount of time leaders/managers are willing to accept for a process outage or disruption.", alias="maximumTolerableDowntime")
    recovery_time_objective: Optional[StrictStr] = Field(default=None, description="[Read-Only] RTO defines the maximum amount of time a system can remain unavailable before there is an unacceptable impact on other systems, supported business processes, and the MTD.", alias="recoveryTimeObjective")
    recovery_point_objective: Optional[StrictStr] = Field(default=None, description="[Read-Only] RPO represents the point in time, prior to a disruption or system outage, to which mission/business data can be recovered (given the most recent backup copy of the data) after an outage. The system data/information owner determines an acceptable RPO in terms of amount of tolerable data loss before unacceptable impact occurs.", alias="recoveryPointObjective")
    business_impact_analysis_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is a Business Impact Analysis in place that identifies critical business processes, MTD, RTO, and RPO?", alias="businessImpactAnalysisRequired")
    business_impact_analysis_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's Business Impact Analysis artifact.", alias="businessImpactAnalysisArtifact")
    incident_response_plan_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is there a Incident Response Plan in place for this system that provides the roadmap for implementing the incident response capability?", alias="incidentResponsePlanRequired")
    incident_response_plan_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's Incident Response Plan artifact.", alias="incidentResponsePlanArtifact")
    disaster_recovery_plan_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Is there a Disaster Recovery Plan in place for this system that addresses information system disruptions that require relocation?", alias="disasterRecoveryPlanRequired")
    disaster_recovery_plan_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's Disaster Recovery Plan artifact.", alias="disasterRecoveryPlanArtifact")
    privacy_threshold_analysis_completed: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicate whether a Privacy Threshold Analysis (PTA) has been performed for this IS.", alias="privacyThresholdAnalysisCompleted")
    privacy_threshold_analysis_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Date in which this sytem's Privacy Threshold Analysis took place.", alias="privacyThresholdAnalysisDate")
    privacy_threshold_analysis_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's Privacy Threshold Analysis artifact.", alias="privacyThresholdAnalysisArtifact")
    privacy_act_system_of_records_notice_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Does this system require a Privacy Act System of Record Notice per DoD 5400.11-R?", alias="privacyActSystemOfRecordsNoticeRequired")
    e_authentication_risk_assessment_required: Optional[StrictBool] = Field(default=None, description="[Read-Only] Indicate whether an E-Authentication Risk Assessment has been performed for the system in accordance with OMB M-04-04.", alias="eAuthenticationRiskAssessmentRequired")
    e_authentication_risk_assessment_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] Date this system's EAuthentication Risk Assessment took place.", alias="eAuthenticationRiskAssessmentDate")
    e_authentication_risk_assessment_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the system's Privacy Threshold Analysis artifact.", alias="eAuthenticationRiskAssessmentArtifact")
    ipv4_only_assets: Optional[StrictInt] = Field(default=None, description="[Read-Only] Identify the total number of assets associated with this boundary that are only on IPv4.", alias="ipv4OnlyAssets")
    ipv6_only_assets: Optional[StrictInt] = Field(default=None, description="[Read-Only] Identify the total number of assets associated with this boundary that are only on IPv6.", alias="ipv6OnlyAssets")
    ipv4_ipv6_dual_stack_assets: Optional[StrictInt] = Field(default=None, description="[Read-Only] Identify the total number of assets associated with this boundary that are operating on IPv4/IPv6 dual-stack.", alias="ipv4Ipv6DualStackAssets")
    total_ip_assets: Optional[StrictInt] = Field(default=None, description="[Read-Only] Total number of IPv4, IPv6, and dual-stack assets identified. Value calculated upon Save action", alias="totalIpAssets")
    originating_organization: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identify the organization that generated the existing authorization package, including (where applicable) individual points of contact.", alias="originatingOrganization")
    system_use_justification: Optional[StrictStr] = Field(default=None, description="[Read-Only] Provide a detailed justification as to why this system should be deployed and used within the requesting organization.", alias="systemUseJustification")
    system_use_justification_artifact: Optional[StrictStr] = Field(default=None, description="[Read-Only] Filename of the System Use Justification Artifact, including the file extension.", alias="systemUseJustificationArtifact")
    authorization_to_use_status: Optional[StrictStr] = Field(default=None, description="[Read-Only] The system's Authority to Use Status.", alias="authorizationToUseStatus")
    reciprocity_acceptance_status: Optional[StrictStr] = Field(default=None, description="[Read-Only] The system's Reciprocity Acceptance Status. Army IC only.", alias="reciprocityAcceptanceStatus")
    use_authorization_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The system's Use Authorization Date.", alias="useAuthorizationDate")
    reciprocity_acceptance_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The system's Reciprocity Acceptance Date. Army IC only.", alias="reciprocityAcceptanceDate")
    use_authorization_termination_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The system's Use Authorization Termination Date.", alias="useAuthorizationTerminationDate")
    reciprocity_acceptance_termination_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The system's Reciprocity Acceptance Termination Date. Army IC only.", alias="reciprocityAcceptanceTerminationDate")
    terms_conditions_for_use_summary: Optional[StrictStr] = Field(default=None, description="[Read-Only] The system's Terms / Conditions for Use Summary.", alias="termsConditionsForUseSummary")
    terms_conditions_for_reciprocity_summary: Optional[StrictStr] = Field(default=None, description="[Read-Only] The system's Terms / Conditions for Reciprocity Summary. Army IC only.", alias="termsConditionsForReciprocitySummary")
    primary_mission_essential_function: Optional[StrictBool] = Field(default=None, description="[Read-Only] Annotate 'Yes' if your system performs a Primary Mission Essential Function. VA only", alias="primaryMissionEssentialFunction")
    pmef_description: Optional[StrictStr] = Field(default=None, description="[Read-Only] The FIPS category \"Availability\" of this system must be marked \"High\" if the system is listed as a PMEF, and the RTO must be 12 hours or shorter. VA only", alias="pmefDescription")
    mission_essential_function: Optional[StrictBool] = Field(default=None, description="[Read-Only] Annotate \"Yes\" if your system performs a Mission Essential Function. VA only.", alias="missionEssentialFunction")
    mef_description: Optional[StrictStr] = Field(default=None, description="[Read-Only] MEFs are listed in the OIT Continuity Plan. VA Handbook 0322 will be updated with these MEFs. VA only.", alias="mefDescription")
    administration: Optional[StrictStr] = Field(default=None, description="[Read-Only] Identify the administration that this information system supports. VA only.")
    administration_other_justification: Optional[StrictStr] = Field(default=None, description="[Read-Only] If the Administration field is set to \"Other\", this free-text field becomes required. VA only.", alias="administrationOtherJustification")
    atc_iatc_granted_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The date the Authority to Connect (ATC) or Interim Authority to Connect (IATC) was granted.", alias="atcIatcGrantedDate")
    atc_iatc_expiration_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The date the Authority to Connect (ATC) or Interim Authority to Connect (IATC) expires.", alias="atcIatcExpirationDate")
    atc_iatc_pending_items: Optional[StrictStr] = Field(default=None, description="[Read-Only] List of pending items required for the Authority to Connect (ATC) or Interim Authority to Connect (IATC).", alias="atcIatcPendingItems")
    pending_items_due_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The date the pending items are due.", alias="pendingItemsDueDate")
    dodinn_system_id: Optional[StrictBool] = Field(default=None, description="[Read-Only] A DoDIN-N system is all encompassing for the following: Navy networks, systems, sites, corporate assets, centrally managed programs, programs of record, standalone, excepted, enterprise, non-enterprise, cloud, NIPRNET, SIPRNET, GENSER Classified, Defense Research Engineering Network (DREN), Secret DREN (SDREN), connected to the DoDIN or through commercial lines that are owned, operated, leased, contracted, maintained, managed, administered, house Navy data, house Navy personally identifiable information, house Navy protected health information, or controlled unclassified information. The DoDIN-N also includes commercial information technology, weapons systems/program manager/program of record/platform information technology, and operational technology. Navy only.", alias="dodinnSystemId")
    authorization_to_use_connect_status: Optional[StrictStr] = Field(default=None, description="[Read-Only] FCC CRMA/NAO approval for DoDIN-N operations. Navy only.", alias="authorizationToUseConnectStatus")
    use_connect_authorization_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The Use/Connect Authorization Date. Navy only.", alias="useConnectAuthorizationDate")
    use_connect_authorization_termination_date: Optional[StrictInt] = Field(default=None, description="[Read-Only] The Use/Connect Authorization Termination Date Navy only.", alias="useConnectAuthorizationTerminationDate")
    terms_conditions_for_use_connect_summary: Optional[StrictStr] = Field(default=None, description="[Read-Only] Provide a description (if applicable) of any specific limitations or restrictions placed on the operation of the information system as part of the authorization to use/connect decision. Navy only.", alias="termsConditionsForUseConnectSummary")
    workstations: Optional[StrictInt] = Field(default=None, description="[Read-Only] Identify the number of System Workstations. This data will be used for the National Industrial Security System (NISS) facility categorization. If field is not applicable enter \"0\". NISP only.")
    servers: Optional[StrictInt] = Field(default=None, description="[Read-Only] Identify the number of System Servers. This data will be used for the National Industrial Security System (NISS) facility categorization. If field is not applicable enter \"0\". NISP only.")
    connectivity_ccsd: Optional[List[ConnectivityCcsd]] = Field(default=None, alias="connectivityCcsd")
    __properties: ClassVar[List[str]] = ["registrationCompletionDate", "systemLifeCycleAcquisitionPhase", "specialType", "specialTypeDescription", "missionPortfolio", "isNNPI", "isRBC", "isWaiver", "programOffice", "vramId", "systemId", "policy", "registrationType", "name", "acronym", "description", "instance", "owningOrganization", "secondaryOrganization", "versionReleaseNo", "systemType", "isNSS", "isPublicFacing", "coamsId", "isTypeAuthorization", "ditprId", "apmsId", "vasiId", "authorizationStatus", "authorizationDate", "authorizationTerminationDate", "authorizationLength", "securityControlsAssessorExecutiveSummary", "riskReviewExecutiveSummary", "termsForAuth", "securityPlanApprovalStatus", "securityPlanApprovalDate", "missionCriticality", "geographicalAssociation", "systemOwnershipControlled", "governingMissionArea", "primaryFunctionalArea", "secondaryFunctionalArea", "primaryControlSet", "confidentiality", "integrity", "availability", "appliedOverlays", "appliedStigs", "rmfActivity", "crossDomainTicket", "ditprDonId", "mac", "dodConfidentiality", "contingencyPlanRequired", "contingencyPlanArtifact", "contingencyPlanTested", "contingencyPlanTestDate", "securityReviewRequired", "securityReviewCompleted", "securityReviewCompletionDate", "nextSecurityReviewDueDate", "hasOpenPoamItem", "hasOpenPoamItem90to120PastScheduledCompletionDate", "hasOpenPoamItem120PlusPastScheudledCompletionDate", "impact", "hasCUI", "hasPII", "hasPHI", "ppsmRegistrationRequired", "ppsmRegistryNumber", "ppsmRegistrationExemptionJustification", "interconnectedInformationSystemAndIdentifiers", "privacyImpactAssessmentRequired", "privacyImpactAssessmentStatus", "privacyImpactAssessmentDate", "privacyImpactAssessmentArtifact", "userDefinedField1", "userDefinedField2", "userDefinedField3", "userDefinedField4", "userDefinedField5", "currentRmfLifecycleStep", "otherInformation", "reportsForScorecard", "highestSystemDataClassification", "overallClassification", "isHVA", "isFinancialManagement", "isReciprocity", "reciprocityExemption", "cloudComputing", "cloudType", "authorizationToConnectStatus", "isSaaS", "isPaaS", "isIaaS", "otherServiceModels", "needDate", "overallRiskScore", "isHRR", "connectivityAuthorizationDate", "connectivityAuthorizationTerminationDate", "systemDevelopmentLifeCycle", "isFISMAReportable", "groupTagging", "groupTagDescriptions", "dadmsId", "dadmsExpirationDate", "enclaveConnectivity", "environmentType", "navyCommonControlProvider", "navyCloudBroker", "cloudBrokerEmassId", "cloudBrokerProvisionalAuthorizationAtd", "navyJointAuthorization", "nmciNgenClins", "enterpriseLocations", "whitelistId", "whitelistInventory", "acquisitionCategory", "softwareCategory", "cybersecurityServiceProvider", "cybersecurityServiceProviderExceptionJustification", "maximumTolerableDowntime", "recoveryTimeObjective", "recoveryPointObjective", "businessImpactAnalysisRequired", "businessImpactAnalysisArtifact", "incidentResponsePlanRequired", "incidentResponsePlanArtifact", "disasterRecoveryPlanRequired", "disasterRecoveryPlanArtifact", "privacyThresholdAnalysisCompleted", "privacyThresholdAnalysisDate", "privacyThresholdAnalysisArtifact", "privacyActSystemOfRecordsNoticeRequired", "eAuthenticationRiskAssessmentRequired", "eAuthenticationRiskAssessmentDate", "eAuthenticationRiskAssessmentArtifact", "ipv4OnlyAssets", "ipv6OnlyAssets", "ipv4Ipv6DualStackAssets", "totalIpAssets", "originatingOrganization", "systemUseJustification", "systemUseJustificationArtifact", "authorizationToUseStatus", "reciprocityAcceptanceStatus", "useAuthorizationDate", "reciprocityAcceptanceDate", "useAuthorizationTerminationDate", "reciprocityAcceptanceTerminationDate", "termsConditionsForUseSummary", "termsConditionsForReciprocitySummary", "primaryMissionEssentialFunction", "pmefDescription", "missionEssentialFunction", "mefDescription", "administration", "administrationOtherJustification", "atcIatcGrantedDate", "atcIatcExpirationDate", "atcIatcPendingItems", "pendingItemsDueDate", "dodinnSystemId", "authorizationToUseConnectStatus", "useConnectAuthorizationDate", "useConnectAuthorizationTerminationDate", "termsConditionsForUseConnectSummary", "workstations", "servers", "connectivityCcsd"]

    @field_validator('policy')
    def policy_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['RMF', 'DIACAP']):
            raise ValueError("must be one of enum values ('RMF', 'DIACAP')")
        return value

    @field_validator('registration_type')
    def registration_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Assess and Authorize', 'Assess Only', 'Guest', 'Regular', 'Functional', 'Cloud Service Provider', 'Authorization To Use']):
            raise ValueError("must be one of enum values ('Assess and Authorize', 'Assess Only', 'Guest', 'Regular', 'Functional', 'Cloud Service Provider', 'Authorization To Use')")
        return value

    @field_validator('system_type')
    def system_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['IS Major Application', 'IS Enclave', 'Platform IT', 'Platform IT System', 'Platform IT Interconnection', 'AIS Application', 'Outsourced IT-Based Process (DoD-controlled)', 'Enclave', 'Outsourced IT-Based Process (service provider shared)']):
            raise ValueError("must be one of enum values ('IS Major Application', 'IS Enclave', 'Platform IT', 'Platform IT System', 'Platform IT Interconnection', 'AIS Application', 'Outsourced IT-Based Process (DoD-controlled)', 'Enclave', 'Outsourced IT-Based Process (service provider shared)')")
        return value

    @field_validator('security_plan_approval_status')
    def security_plan_approval_status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Approved', 'Not Yet Approved', 'Denied']):
            raise ValueError("must be one of enum values ('Approved', 'Not Yet Approved', 'Denied')")
        return value

    @field_validator('primary_control_set')
    def primary_control_set_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['NIST SP 800-53 Revision 4', 'DoDI 8500.2']):
            raise ValueError("must be one of enum values ('NIST SP 800-53 Revision 4', 'DoDI 8500.2')")
        return value

    @field_validator('confidentiality')
    def confidentiality_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['High', 'Moderate', 'Low']):
            raise ValueError("must be one of enum values ('High', 'Moderate', 'Low')")
        return value

    @field_validator('integrity')
    def integrity_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['High', 'Moderate', 'Low']):
            raise ValueError("must be one of enum values ('High', 'Moderate', 'Low')")
        return value

    @field_validator('availability')
    def availability_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['High', 'Moderate', 'Low']):
            raise ValueError("must be one of enum values ('High', 'Moderate', 'Low')")
        return value

    @field_validator('mac')
    def mac_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['I', 'II', 'III']):
            raise ValueError("must be one of enum values ('I', 'II', 'III')")
        return value

    @field_validator('dod_confidentiality')
    def dod_confidentiality_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Public', 'Sensitive', 'Classified']):
            raise ValueError("must be one of enum values ('Public', 'Sensitive', 'Classified')")
        return value

    @field_validator('impact')
    def impact_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Low', 'Moderate', 'High']):
            raise ValueError("must be one of enum values ('Low', 'Moderate', 'High')")
        return value

    @field_validator('privacy_impact_assessment_status')
    def privacy_impact_assessment_status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Not Started', 'In Progress', 'Completed']):
            raise ValueError("must be one of enum values ('Not Started', 'In Progress', 'Completed')")
        return value

    @field_validator('current_rmf_lifecycle_step')
    def current_rmf_lifecycle_step_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['1 - Categorize', '2 - Select', '3 - Implement', '4 - Assess', '5 - Authorize', '6 - Monitor']):
            raise ValueError("must be one of enum values ('1 - Categorize', '2 - Select', '3 - Implement', '4 - Assess', '5 - Authorize', '6 - Monitor')")
        return value

    @field_validator('cloud_type')
    def cloud_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Hybrid', 'Private', 'Public']):
            raise ValueError("must be one of enum values ('Hybrid', 'Private', 'Public')")
        return value

    @field_validator('authorization_to_connect_status')
    def authorization_to_connect_status_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Authority to Connect (ATC)', 'Denial of Authority to Connect (DATC)', 'Not Yet Connected', 'Decommissioned']):
            raise ValueError("must be one of enum values ('Authority to Connect (ATC)', 'Denial of Authority to Connect (DATC)', 'Not Yet Connected', 'Decommissioned')")
        return value

    @field_validator('maximum_tolerable_downtime')
    def maximum_tolerable_downtime_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Immediate', 'Less than 2 hours', 'Less than 4 hours', 'Less than 6 hours']):
            raise ValueError("must be one of enum values ('Immediate', 'Less than 2 hours', 'Less than 4 hours', 'Less than 6 hours')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Systems from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in connectivity_ccsd (list)
        _items = []
        if self.connectivity_ccsd:
            for _item_connectivity_ccsd in self.connectivity_ccsd:
                if _item_connectivity_ccsd:
                    _items.append(_item_connectivity_ccsd.to_dict())
            _dict['connectivityCcsd'] = _items
        # set to None if registration_completion_date (nullable) is None
        # and model_fields_set contains the field
        if self.registration_completion_date is None and "registration_completion_date" in self.model_fields_set:
            _dict['registrationCompletionDate'] = None

        # set to None if system_life_cycle_acquisition_phase (nullable) is None
        # and model_fields_set contains the field
        if self.system_life_cycle_acquisition_phase is None and "system_life_cycle_acquisition_phase" in self.model_fields_set:
            _dict['systemLifeCycleAcquisitionPhase'] = None

        # set to None if special_type (nullable) is None
        # and model_fields_set contains the field
        if self.special_type is None and "special_type" in self.model_fields_set:
            _dict['specialType'] = None

        # set to None if special_type_description (nullable) is None
        # and model_fields_set contains the field
        if self.special_type_description is None and "special_type_description" in self.model_fields_set:
            _dict['specialTypeDescription'] = None

        # set to None if mission_portfolio (nullable) is None
        # and model_fields_set contains the field
        if self.mission_portfolio is None and "mission_portfolio" in self.model_fields_set:
            _dict['missionPortfolio'] = None

        # set to None if is_nnpi (nullable) is None
        # and model_fields_set contains the field
        if self.is_nnpi is None and "is_nnpi" in self.model_fields_set:
            _dict['isNNPI'] = None

        # set to None if is_rbc (nullable) is None
        # and model_fields_set contains the field
        if self.is_rbc is None and "is_rbc" in self.model_fields_set:
            _dict['isRBC'] = None

        # set to None if is_waiver (nullable) is None
        # and model_fields_set contains the field
        if self.is_waiver is None and "is_waiver" in self.model_fields_set:
            _dict['isWaiver'] = None

        # set to None if program_office (nullable) is None
        # and model_fields_set contains the field
        if self.program_office is None and "program_office" in self.model_fields_set:
            _dict['programOffice'] = None

        # set to None if vram_id (nullable) is None
        # and model_fields_set contains the field
        if self.vram_id is None and "vram_id" in self.model_fields_set:
            _dict['vramId'] = None

        # set to None if policy (nullable) is None
        # and model_fields_set contains the field
        if self.policy is None and "policy" in self.model_fields_set:
            _dict['policy'] = None

        # set to None if registration_type (nullable) is None
        # and model_fields_set contains the field
        if self.registration_type is None and "registration_type" in self.model_fields_set:
            _dict['registrationType'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        # set to None if acronym (nullable) is None
        # and model_fields_set contains the field
        if self.acronym is None and "acronym" in self.model_fields_set:
            _dict['acronym'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if instance (nullable) is None
        # and model_fields_set contains the field
        if self.instance is None and "instance" in self.model_fields_set:
            _dict['instance'] = None

        # set to None if owning_organization (nullable) is None
        # and model_fields_set contains the field
        if self.owning_organization is None and "owning_organization" in self.model_fields_set:
            _dict['owningOrganization'] = None

        # set to None if secondary_organization (nullable) is None
        # and model_fields_set contains the field
        if self.secondary_organization is None and "secondary_organization" in self.model_fields_set:
            _dict['secondaryOrganization'] = None

        # set to None if version_release_no (nullable) is None
        # and model_fields_set contains the field
        if self.version_release_no is None and "version_release_no" in self.model_fields_set:
            _dict['versionReleaseNo'] = None

        # set to None if system_type (nullable) is None
        # and model_fields_set contains the field
        if self.system_type is None and "system_type" in self.model_fields_set:
            _dict['systemType'] = None

        # set to None if is_nss (nullable) is None
        # and model_fields_set contains the field
        if self.is_nss is None and "is_nss" in self.model_fields_set:
            _dict['isNSS'] = None

        # set to None if is_public_facing (nullable) is None
        # and model_fields_set contains the field
        if self.is_public_facing is None and "is_public_facing" in self.model_fields_set:
            _dict['isPublicFacing'] = None

        # set to None if coams_id (nullable) is None
        # and model_fields_set contains the field
        if self.coams_id is None and "coams_id" in self.model_fields_set:
            _dict['coamsId'] = None

        # set to None if is_type_authorization (nullable) is None
        # and model_fields_set contains the field
        if self.is_type_authorization is None and "is_type_authorization" in self.model_fields_set:
            _dict['isTypeAuthorization'] = None

        # set to None if apms_id (nullable) is None
        # and model_fields_set contains the field
        if self.apms_id is None and "apms_id" in self.model_fields_set:
            _dict['apmsId'] = None

        # set to None if vasi_id (nullable) is None
        # and model_fields_set contains the field
        if self.vasi_id is None and "vasi_id" in self.model_fields_set:
            _dict['vasiId'] = None

        # set to None if authorization_status (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_status is None and "authorization_status" in self.model_fields_set:
            _dict['authorizationStatus'] = None

        # set to None if authorization_date (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_date is None and "authorization_date" in self.model_fields_set:
            _dict['authorizationDate'] = None

        # set to None if authorization_termination_date (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_termination_date is None and "authorization_termination_date" in self.model_fields_set:
            _dict['authorizationTerminationDate'] = None

        # set to None if authorization_length (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_length is None and "authorization_length" in self.model_fields_set:
            _dict['authorizationLength'] = None

        # set to None if security_controls_assessor_executive_summary (nullable) is None
        # and model_fields_set contains the field
        if self.security_controls_assessor_executive_summary is None and "security_controls_assessor_executive_summary" in self.model_fields_set:
            _dict['securityControlsAssessorExecutiveSummary'] = None

        # set to None if risk_review_executive_summary (nullable) is None
        # and model_fields_set contains the field
        if self.risk_review_executive_summary is None and "risk_review_executive_summary" in self.model_fields_set:
            _dict['riskReviewExecutiveSummary'] = None

        # set to None if terms_for_auth (nullable) is None
        # and model_fields_set contains the field
        if self.terms_for_auth is None and "terms_for_auth" in self.model_fields_set:
            _dict['termsForAuth'] = None

        # set to None if security_plan_approval_status (nullable) is None
        # and model_fields_set contains the field
        if self.security_plan_approval_status is None and "security_plan_approval_status" in self.model_fields_set:
            _dict['securityPlanApprovalStatus'] = None

        # set to None if security_plan_approval_date (nullable) is None
        # and model_fields_set contains the field
        if self.security_plan_approval_date is None and "security_plan_approval_date" in self.model_fields_set:
            _dict['securityPlanApprovalDate'] = None

        # set to None if mission_criticality (nullable) is None
        # and model_fields_set contains the field
        if self.mission_criticality is None and "mission_criticality" in self.model_fields_set:
            _dict['missionCriticality'] = None

        # set to None if geographical_association (nullable) is None
        # and model_fields_set contains the field
        if self.geographical_association is None and "geographical_association" in self.model_fields_set:
            _dict['geographicalAssociation'] = None

        # set to None if system_ownership_controlled (nullable) is None
        # and model_fields_set contains the field
        if self.system_ownership_controlled is None and "system_ownership_controlled" in self.model_fields_set:
            _dict['systemOwnershipControlled'] = None

        # set to None if governing_mission_area (nullable) is None
        # and model_fields_set contains the field
        if self.governing_mission_area is None and "governing_mission_area" in self.model_fields_set:
            _dict['governingMissionArea'] = None

        # set to None if primary_functional_area (nullable) is None
        # and model_fields_set contains the field
        if self.primary_functional_area is None and "primary_functional_area" in self.model_fields_set:
            _dict['primaryFunctionalArea'] = None

        # set to None if secondary_functional_area (nullable) is None
        # and model_fields_set contains the field
        if self.secondary_functional_area is None and "secondary_functional_area" in self.model_fields_set:
            _dict['secondaryFunctionalArea'] = None

        # set to None if primary_control_set (nullable) is None
        # and model_fields_set contains the field
        if self.primary_control_set is None and "primary_control_set" in self.model_fields_set:
            _dict['primaryControlSet'] = None

        # set to None if confidentiality (nullable) is None
        # and model_fields_set contains the field
        if self.confidentiality is None and "confidentiality" in self.model_fields_set:
            _dict['confidentiality'] = None

        # set to None if integrity (nullable) is None
        # and model_fields_set contains the field
        if self.integrity is None and "integrity" in self.model_fields_set:
            _dict['integrity'] = None

        # set to None if availability (nullable) is None
        # and model_fields_set contains the field
        if self.availability is None and "availability" in self.model_fields_set:
            _dict['availability'] = None

        # set to None if applied_overlays (nullable) is None
        # and model_fields_set contains the field
        if self.applied_overlays is None and "applied_overlays" in self.model_fields_set:
            _dict['appliedOverlays'] = None

        # set to None if applied_stigs (nullable) is None
        # and model_fields_set contains the field
        if self.applied_stigs is None and "applied_stigs" in self.model_fields_set:
            _dict['appliedStigs'] = None

        # set to None if rmf_activity (nullable) is None
        # and model_fields_set contains the field
        if self.rmf_activity is None and "rmf_activity" in self.model_fields_set:
            _dict['rmfActivity'] = None

        # set to None if cross_domain_ticket (nullable) is None
        # and model_fields_set contains the field
        if self.cross_domain_ticket is None and "cross_domain_ticket" in self.model_fields_set:
            _dict['crossDomainTicket'] = None

        # set to None if ditpr_don_id (nullable) is None
        # and model_fields_set contains the field
        if self.ditpr_don_id is None and "ditpr_don_id" in self.model_fields_set:
            _dict['ditprDonId'] = None

        # set to None if mac (nullable) is None
        # and model_fields_set contains the field
        if self.mac is None and "mac" in self.model_fields_set:
            _dict['mac'] = None

        # set to None if dod_confidentiality (nullable) is None
        # and model_fields_set contains the field
        if self.dod_confidentiality is None and "dod_confidentiality" in self.model_fields_set:
            _dict['dodConfidentiality'] = None

        # set to None if contingency_plan_required (nullable) is None
        # and model_fields_set contains the field
        if self.contingency_plan_required is None and "contingency_plan_required" in self.model_fields_set:
            _dict['contingencyPlanRequired'] = None

        # set to None if contingency_plan_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.contingency_plan_artifact is None and "contingency_plan_artifact" in self.model_fields_set:
            _dict['contingencyPlanArtifact'] = None

        # set to None if contingency_plan_tested (nullable) is None
        # and model_fields_set contains the field
        if self.contingency_plan_tested is None and "contingency_plan_tested" in self.model_fields_set:
            _dict['contingencyPlanTested'] = None

        # set to None if contingency_plan_test_date (nullable) is None
        # and model_fields_set contains the field
        if self.contingency_plan_test_date is None and "contingency_plan_test_date" in self.model_fields_set:
            _dict['contingencyPlanTestDate'] = None

        # set to None if security_review_required (nullable) is None
        # and model_fields_set contains the field
        if self.security_review_required is None and "security_review_required" in self.model_fields_set:
            _dict['securityReviewRequired'] = None

        # set to None if security_review_completed (nullable) is None
        # and model_fields_set contains the field
        if self.security_review_completed is None and "security_review_completed" in self.model_fields_set:
            _dict['securityReviewCompleted'] = None

        # set to None if security_review_completion_date (nullable) is None
        # and model_fields_set contains the field
        if self.security_review_completion_date is None and "security_review_completion_date" in self.model_fields_set:
            _dict['securityReviewCompletionDate'] = None

        # set to None if next_security_review_due_date (nullable) is None
        # and model_fields_set contains the field
        if self.next_security_review_due_date is None and "next_security_review_due_date" in self.model_fields_set:
            _dict['nextSecurityReviewDueDate'] = None

        # set to None if has_open_poam_item (nullable) is None
        # and model_fields_set contains the field
        if self.has_open_poam_item is None and "has_open_poam_item" in self.model_fields_set:
            _dict['hasOpenPoamItem'] = None

        # set to None if has_open_poam_item90to120_past_scheduled_completion_date (nullable) is None
        # and model_fields_set contains the field
        if self.has_open_poam_item90to120_past_scheduled_completion_date is None and "has_open_poam_item90to120_past_scheduled_completion_date" in self.model_fields_set:
            _dict['hasOpenPoamItem90to120PastScheduledCompletionDate'] = None

        # set to None if has_open_poam_item120_plus_past_scheudled_completion_date (nullable) is None
        # and model_fields_set contains the field
        if self.has_open_poam_item120_plus_past_scheudled_completion_date is None and "has_open_poam_item120_plus_past_scheudled_completion_date" in self.model_fields_set:
            _dict['hasOpenPoamItem120PlusPastScheudledCompletionDate'] = None

        # set to None if impact (nullable) is None
        # and model_fields_set contains the field
        if self.impact is None and "impact" in self.model_fields_set:
            _dict['impact'] = None

        # set to None if has_cui (nullable) is None
        # and model_fields_set contains the field
        if self.has_cui is None and "has_cui" in self.model_fields_set:
            _dict['hasCUI'] = None

        # set to None if has_pii (nullable) is None
        # and model_fields_set contains the field
        if self.has_pii is None and "has_pii" in self.model_fields_set:
            _dict['hasPII'] = None

        # set to None if has_phi (nullable) is None
        # and model_fields_set contains the field
        if self.has_phi is None and "has_phi" in self.model_fields_set:
            _dict['hasPHI'] = None

        # set to None if ppsm_registration_required (nullable) is None
        # and model_fields_set contains the field
        if self.ppsm_registration_required is None and "ppsm_registration_required" in self.model_fields_set:
            _dict['ppsmRegistrationRequired'] = None

        # set to None if ppsm_registry_number (nullable) is None
        # and model_fields_set contains the field
        if self.ppsm_registry_number is None and "ppsm_registry_number" in self.model_fields_set:
            _dict['ppsmRegistryNumber'] = None

        # set to None if ppsm_registration_exemption_justification (nullable) is None
        # and model_fields_set contains the field
        if self.ppsm_registration_exemption_justification is None and "ppsm_registration_exemption_justification" in self.model_fields_set:
            _dict['ppsmRegistrationExemptionJustification'] = None

        # set to None if interconnected_information_system_and_identifiers (nullable) is None
        # and model_fields_set contains the field
        if self.interconnected_information_system_and_identifiers is None and "interconnected_information_system_and_identifiers" in self.model_fields_set:
            _dict['interconnectedInformationSystemAndIdentifiers'] = None

        # set to None if privacy_impact_assessment_required (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_impact_assessment_required is None and "privacy_impact_assessment_required" in self.model_fields_set:
            _dict['privacyImpactAssessmentRequired'] = None

        # set to None if privacy_impact_assessment_status (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_impact_assessment_status is None and "privacy_impact_assessment_status" in self.model_fields_set:
            _dict['privacyImpactAssessmentStatus'] = None

        # set to None if privacy_impact_assessment_date (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_impact_assessment_date is None and "privacy_impact_assessment_date" in self.model_fields_set:
            _dict['privacyImpactAssessmentDate'] = None

        # set to None if privacy_impact_assessment_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_impact_assessment_artifact is None and "privacy_impact_assessment_artifact" in self.model_fields_set:
            _dict['privacyImpactAssessmentArtifact'] = None

        # set to None if user_defined_field1 (nullable) is None
        # and model_fields_set contains the field
        if self.user_defined_field1 is None and "user_defined_field1" in self.model_fields_set:
            _dict['userDefinedField1'] = None

        # set to None if user_defined_field2 (nullable) is None
        # and model_fields_set contains the field
        if self.user_defined_field2 is None and "user_defined_field2" in self.model_fields_set:
            _dict['userDefinedField2'] = None

        # set to None if user_defined_field3 (nullable) is None
        # and model_fields_set contains the field
        if self.user_defined_field3 is None and "user_defined_field3" in self.model_fields_set:
            _dict['userDefinedField3'] = None

        # set to None if user_defined_field4 (nullable) is None
        # and model_fields_set contains the field
        if self.user_defined_field4 is None and "user_defined_field4" in self.model_fields_set:
            _dict['userDefinedField4'] = None

        # set to None if user_defined_field5 (nullable) is None
        # and model_fields_set contains the field
        if self.user_defined_field5 is None and "user_defined_field5" in self.model_fields_set:
            _dict['userDefinedField5'] = None

        # set to None if current_rmf_lifecycle_step (nullable) is None
        # and model_fields_set contains the field
        if self.current_rmf_lifecycle_step is None and "current_rmf_lifecycle_step" in self.model_fields_set:
            _dict['currentRmfLifecycleStep'] = None

        # set to None if other_information (nullable) is None
        # and model_fields_set contains the field
        if self.other_information is None and "other_information" in self.model_fields_set:
            _dict['otherInformation'] = None

        # set to None if reports_for_scorecard (nullable) is None
        # and model_fields_set contains the field
        if self.reports_for_scorecard is None and "reports_for_scorecard" in self.model_fields_set:
            _dict['reportsForScorecard'] = None

        # set to None if highest_system_data_classification (nullable) is None
        # and model_fields_set contains the field
        if self.highest_system_data_classification is None and "highest_system_data_classification" in self.model_fields_set:
            _dict['highestSystemDataClassification'] = None

        # set to None if overall_classification (nullable) is None
        # and model_fields_set contains the field
        if self.overall_classification is None and "overall_classification" in self.model_fields_set:
            _dict['overallClassification'] = None

        # set to None if is_hva (nullable) is None
        # and model_fields_set contains the field
        if self.is_hva is None and "is_hva" in self.model_fields_set:
            _dict['isHVA'] = None

        # set to None if is_financial_management (nullable) is None
        # and model_fields_set contains the field
        if self.is_financial_management is None and "is_financial_management" in self.model_fields_set:
            _dict['isFinancialManagement'] = None

        # set to None if is_reciprocity (nullable) is None
        # and model_fields_set contains the field
        if self.is_reciprocity is None and "is_reciprocity" in self.model_fields_set:
            _dict['isReciprocity'] = None

        # set to None if reciprocity_exemption (nullable) is None
        # and model_fields_set contains the field
        if self.reciprocity_exemption is None and "reciprocity_exemption" in self.model_fields_set:
            _dict['reciprocityExemption'] = None

        # set to None if cloud_computing (nullable) is None
        # and model_fields_set contains the field
        if self.cloud_computing is None and "cloud_computing" in self.model_fields_set:
            _dict['cloudComputing'] = None

        # set to None if cloud_type (nullable) is None
        # and model_fields_set contains the field
        if self.cloud_type is None and "cloud_type" in self.model_fields_set:
            _dict['cloudType'] = None

        # set to None if authorization_to_connect_status (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_to_connect_status is None and "authorization_to_connect_status" in self.model_fields_set:
            _dict['authorizationToConnectStatus'] = None

        # set to None if is_saa_s (nullable) is None
        # and model_fields_set contains the field
        if self.is_saa_s is None and "is_saa_s" in self.model_fields_set:
            _dict['isSaaS'] = None

        # set to None if is_paa_s (nullable) is None
        # and model_fields_set contains the field
        if self.is_paa_s is None and "is_paa_s" in self.model_fields_set:
            _dict['isPaaS'] = None

        # set to None if is_iaa_s (nullable) is None
        # and model_fields_set contains the field
        if self.is_iaa_s is None and "is_iaa_s" in self.model_fields_set:
            _dict['isIaaS'] = None

        # set to None if other_service_models (nullable) is None
        # and model_fields_set contains the field
        if self.other_service_models is None and "other_service_models" in self.model_fields_set:
            _dict['otherServiceModels'] = None

        # set to None if need_date (nullable) is None
        # and model_fields_set contains the field
        if self.need_date is None and "need_date" in self.model_fields_set:
            _dict['needDate'] = None

        # set to None if overall_risk_score (nullable) is None
        # and model_fields_set contains the field
        if self.overall_risk_score is None and "overall_risk_score" in self.model_fields_set:
            _dict['overallRiskScore'] = None

        # set to None if is_hrr (nullable) is None
        # and model_fields_set contains the field
        if self.is_hrr is None and "is_hrr" in self.model_fields_set:
            _dict['isHRR'] = None

        # set to None if connectivity_authorization_date (nullable) is None
        # and model_fields_set contains the field
        if self.connectivity_authorization_date is None and "connectivity_authorization_date" in self.model_fields_set:
            _dict['connectivityAuthorizationDate'] = None

        # set to None if connectivity_authorization_termination_date (nullable) is None
        # and model_fields_set contains the field
        if self.connectivity_authorization_termination_date is None and "connectivity_authorization_termination_date" in self.model_fields_set:
            _dict['connectivityAuthorizationTerminationDate'] = None

        # set to None if system_development_life_cycle (nullable) is None
        # and model_fields_set contains the field
        if self.system_development_life_cycle is None and "system_development_life_cycle" in self.model_fields_set:
            _dict['systemDevelopmentLifeCycle'] = None

        # set to None if is_fisma_reportable (nullable) is None
        # and model_fields_set contains the field
        if self.is_fisma_reportable is None and "is_fisma_reportable" in self.model_fields_set:
            _dict['isFISMAReportable'] = None

        # set to None if group_tagging (nullable) is None
        # and model_fields_set contains the field
        if self.group_tagging is None and "group_tagging" in self.model_fields_set:
            _dict['groupTagging'] = None

        # set to None if group_tag_descriptions (nullable) is None
        # and model_fields_set contains the field
        if self.group_tag_descriptions is None and "group_tag_descriptions" in self.model_fields_set:
            _dict['groupTagDescriptions'] = None

        # set to None if dadms_id (nullable) is None
        # and model_fields_set contains the field
        if self.dadms_id is None and "dadms_id" in self.model_fields_set:
            _dict['dadmsId'] = None

        # set to None if dadms_expiration_date (nullable) is None
        # and model_fields_set contains the field
        if self.dadms_expiration_date is None and "dadms_expiration_date" in self.model_fields_set:
            _dict['dadmsExpirationDate'] = None

        # set to None if enclave_connectivity (nullable) is None
        # and model_fields_set contains the field
        if self.enclave_connectivity is None and "enclave_connectivity" in self.model_fields_set:
            _dict['enclaveConnectivity'] = None

        # set to None if environment_type (nullable) is None
        # and model_fields_set contains the field
        if self.environment_type is None and "environment_type" in self.model_fields_set:
            _dict['environmentType'] = None

        # set to None if navy_common_control_provider (nullable) is None
        # and model_fields_set contains the field
        if self.navy_common_control_provider is None and "navy_common_control_provider" in self.model_fields_set:
            _dict['navyCommonControlProvider'] = None

        # set to None if navy_cloud_broker (nullable) is None
        # and model_fields_set contains the field
        if self.navy_cloud_broker is None and "navy_cloud_broker" in self.model_fields_set:
            _dict['navyCloudBroker'] = None

        # set to None if cloud_broker_emass_id (nullable) is None
        # and model_fields_set contains the field
        if self.cloud_broker_emass_id is None and "cloud_broker_emass_id" in self.model_fields_set:
            _dict['cloudBrokerEmassId'] = None

        # set to None if cloud_broker_provisional_authorization_atd (nullable) is None
        # and model_fields_set contains the field
        if self.cloud_broker_provisional_authorization_atd is None and "cloud_broker_provisional_authorization_atd" in self.model_fields_set:
            _dict['cloudBrokerProvisionalAuthorizationAtd'] = None

        # set to None if navy_joint_authorization (nullable) is None
        # and model_fields_set contains the field
        if self.navy_joint_authorization is None and "navy_joint_authorization" in self.model_fields_set:
            _dict['navyJointAuthorization'] = None

        # set to None if nmci_ngen_clins (nullable) is None
        # and model_fields_set contains the field
        if self.nmci_ngen_clins is None and "nmci_ngen_clins" in self.model_fields_set:
            _dict['nmciNgenClins'] = None

        # set to None if enterprise_locations (nullable) is None
        # and model_fields_set contains the field
        if self.enterprise_locations is None and "enterprise_locations" in self.model_fields_set:
            _dict['enterpriseLocations'] = None

        # set to None if whitelist_id (nullable) is None
        # and model_fields_set contains the field
        if self.whitelist_id is None and "whitelist_id" in self.model_fields_set:
            _dict['whitelistId'] = None

        # set to None if whitelist_inventory (nullable) is None
        # and model_fields_set contains the field
        if self.whitelist_inventory is None and "whitelist_inventory" in self.model_fields_set:
            _dict['whitelistInventory'] = None

        # set to None if acquisition_category (nullable) is None
        # and model_fields_set contains the field
        if self.acquisition_category is None and "acquisition_category" in self.model_fields_set:
            _dict['acquisitionCategory'] = None

        # set to None if software_category (nullable) is None
        # and model_fields_set contains the field
        if self.software_category is None and "software_category" in self.model_fields_set:
            _dict['softwareCategory'] = None

        # set to None if cybersecurity_service_provider (nullable) is None
        # and model_fields_set contains the field
        if self.cybersecurity_service_provider is None and "cybersecurity_service_provider" in self.model_fields_set:
            _dict['cybersecurityServiceProvider'] = None

        # set to None if cybersecurity_service_provider_exception_justification (nullable) is None
        # and model_fields_set contains the field
        if self.cybersecurity_service_provider_exception_justification is None and "cybersecurity_service_provider_exception_justification" in self.model_fields_set:
            _dict['cybersecurityServiceProviderExceptionJustification'] = None

        # set to None if maximum_tolerable_downtime (nullable) is None
        # and model_fields_set contains the field
        if self.maximum_tolerable_downtime is None and "maximum_tolerable_downtime" in self.model_fields_set:
            _dict['maximumTolerableDowntime'] = None

        # set to None if recovery_time_objective (nullable) is None
        # and model_fields_set contains the field
        if self.recovery_time_objective is None and "recovery_time_objective" in self.model_fields_set:
            _dict['recoveryTimeObjective'] = None

        # set to None if recovery_point_objective (nullable) is None
        # and model_fields_set contains the field
        if self.recovery_point_objective is None and "recovery_point_objective" in self.model_fields_set:
            _dict['recoveryPointObjective'] = None

        # set to None if business_impact_analysis_required (nullable) is None
        # and model_fields_set contains the field
        if self.business_impact_analysis_required is None and "business_impact_analysis_required" in self.model_fields_set:
            _dict['businessImpactAnalysisRequired'] = None

        # set to None if business_impact_analysis_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.business_impact_analysis_artifact is None and "business_impact_analysis_artifact" in self.model_fields_set:
            _dict['businessImpactAnalysisArtifact'] = None

        # set to None if incident_response_plan_required (nullable) is None
        # and model_fields_set contains the field
        if self.incident_response_plan_required is None and "incident_response_plan_required" in self.model_fields_set:
            _dict['incidentResponsePlanRequired'] = None

        # set to None if incident_response_plan_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.incident_response_plan_artifact is None and "incident_response_plan_artifact" in self.model_fields_set:
            _dict['incidentResponsePlanArtifact'] = None

        # set to None if disaster_recovery_plan_required (nullable) is None
        # and model_fields_set contains the field
        if self.disaster_recovery_plan_required is None and "disaster_recovery_plan_required" in self.model_fields_set:
            _dict['disasterRecoveryPlanRequired'] = None

        # set to None if disaster_recovery_plan_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.disaster_recovery_plan_artifact is None and "disaster_recovery_plan_artifact" in self.model_fields_set:
            _dict['disasterRecoveryPlanArtifact'] = None

        # set to None if privacy_threshold_analysis_completed (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_threshold_analysis_completed is None and "privacy_threshold_analysis_completed" in self.model_fields_set:
            _dict['privacyThresholdAnalysisCompleted'] = None

        # set to None if privacy_threshold_analysis_date (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_threshold_analysis_date is None and "privacy_threshold_analysis_date" in self.model_fields_set:
            _dict['privacyThresholdAnalysisDate'] = None

        # set to None if privacy_threshold_analysis_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_threshold_analysis_artifact is None and "privacy_threshold_analysis_artifact" in self.model_fields_set:
            _dict['privacyThresholdAnalysisArtifact'] = None

        # set to None if privacy_act_system_of_records_notice_required (nullable) is None
        # and model_fields_set contains the field
        if self.privacy_act_system_of_records_notice_required is None and "privacy_act_system_of_records_notice_required" in self.model_fields_set:
            _dict['privacyActSystemOfRecordsNoticeRequired'] = None

        # set to None if e_authentication_risk_assessment_required (nullable) is None
        # and model_fields_set contains the field
        if self.e_authentication_risk_assessment_required is None and "e_authentication_risk_assessment_required" in self.model_fields_set:
            _dict['eAuthenticationRiskAssessmentRequired'] = None

        # set to None if e_authentication_risk_assessment_date (nullable) is None
        # and model_fields_set contains the field
        if self.e_authentication_risk_assessment_date is None and "e_authentication_risk_assessment_date" in self.model_fields_set:
            _dict['eAuthenticationRiskAssessmentDate'] = None

        # set to None if e_authentication_risk_assessment_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.e_authentication_risk_assessment_artifact is None and "e_authentication_risk_assessment_artifact" in self.model_fields_set:
            _dict['eAuthenticationRiskAssessmentArtifact'] = None

        # set to None if ipv4_only_assets (nullable) is None
        # and model_fields_set contains the field
        if self.ipv4_only_assets is None and "ipv4_only_assets" in self.model_fields_set:
            _dict['ipv4OnlyAssets'] = None

        # set to None if ipv6_only_assets (nullable) is None
        # and model_fields_set contains the field
        if self.ipv6_only_assets is None and "ipv6_only_assets" in self.model_fields_set:
            _dict['ipv6OnlyAssets'] = None

        # set to None if ipv4_ipv6_dual_stack_assets (nullable) is None
        # and model_fields_set contains the field
        if self.ipv4_ipv6_dual_stack_assets is None and "ipv4_ipv6_dual_stack_assets" in self.model_fields_set:
            _dict['ipv4Ipv6DualStackAssets'] = None

        # set to None if total_ip_assets (nullable) is None
        # and model_fields_set contains the field
        if self.total_ip_assets is None and "total_ip_assets" in self.model_fields_set:
            _dict['totalIpAssets'] = None

        # set to None if originating_organization (nullable) is None
        # and model_fields_set contains the field
        if self.originating_organization is None and "originating_organization" in self.model_fields_set:
            _dict['originatingOrganization'] = None

        # set to None if system_use_justification (nullable) is None
        # and model_fields_set contains the field
        if self.system_use_justification is None and "system_use_justification" in self.model_fields_set:
            _dict['systemUseJustification'] = None

        # set to None if system_use_justification_artifact (nullable) is None
        # and model_fields_set contains the field
        if self.system_use_justification_artifact is None and "system_use_justification_artifact" in self.model_fields_set:
            _dict['systemUseJustificationArtifact'] = None

        # set to None if authorization_to_use_status (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_to_use_status is None and "authorization_to_use_status" in self.model_fields_set:
            _dict['authorizationToUseStatus'] = None

        # set to None if reciprocity_acceptance_status (nullable) is None
        # and model_fields_set contains the field
        if self.reciprocity_acceptance_status is None and "reciprocity_acceptance_status" in self.model_fields_set:
            _dict['reciprocityAcceptanceStatus'] = None

        # set to None if use_authorization_date (nullable) is None
        # and model_fields_set contains the field
        if self.use_authorization_date is None and "use_authorization_date" in self.model_fields_set:
            _dict['useAuthorizationDate'] = None

        # set to None if reciprocity_acceptance_date (nullable) is None
        # and model_fields_set contains the field
        if self.reciprocity_acceptance_date is None and "reciprocity_acceptance_date" in self.model_fields_set:
            _dict['reciprocityAcceptanceDate'] = None

        # set to None if use_authorization_termination_date (nullable) is None
        # and model_fields_set contains the field
        if self.use_authorization_termination_date is None and "use_authorization_termination_date" in self.model_fields_set:
            _dict['useAuthorizationTerminationDate'] = None

        # set to None if reciprocity_acceptance_termination_date (nullable) is None
        # and model_fields_set contains the field
        if self.reciprocity_acceptance_termination_date is None and "reciprocity_acceptance_termination_date" in self.model_fields_set:
            _dict['reciprocityAcceptanceTerminationDate'] = None

        # set to None if terms_conditions_for_use_summary (nullable) is None
        # and model_fields_set contains the field
        if self.terms_conditions_for_use_summary is None and "terms_conditions_for_use_summary" in self.model_fields_set:
            _dict['termsConditionsForUseSummary'] = None

        # set to None if terms_conditions_for_reciprocity_summary (nullable) is None
        # and model_fields_set contains the field
        if self.terms_conditions_for_reciprocity_summary is None and "terms_conditions_for_reciprocity_summary" in self.model_fields_set:
            _dict['termsConditionsForReciprocitySummary'] = None

        # set to None if primary_mission_essential_function (nullable) is None
        # and model_fields_set contains the field
        if self.primary_mission_essential_function is None and "primary_mission_essential_function" in self.model_fields_set:
            _dict['primaryMissionEssentialFunction'] = None

        # set to None if pmef_description (nullable) is None
        # and model_fields_set contains the field
        if self.pmef_description is None and "pmef_description" in self.model_fields_set:
            _dict['pmefDescription'] = None

        # set to None if mission_essential_function (nullable) is None
        # and model_fields_set contains the field
        if self.mission_essential_function is None and "mission_essential_function" in self.model_fields_set:
            _dict['missionEssentialFunction'] = None

        # set to None if mef_description (nullable) is None
        # and model_fields_set contains the field
        if self.mef_description is None and "mef_description" in self.model_fields_set:
            _dict['mefDescription'] = None

        # set to None if administration (nullable) is None
        # and model_fields_set contains the field
        if self.administration is None and "administration" in self.model_fields_set:
            _dict['administration'] = None

        # set to None if administration_other_justification (nullable) is None
        # and model_fields_set contains the field
        if self.administration_other_justification is None and "administration_other_justification" in self.model_fields_set:
            _dict['administrationOtherJustification'] = None

        # set to None if atc_iatc_granted_date (nullable) is None
        # and model_fields_set contains the field
        if self.atc_iatc_granted_date is None and "atc_iatc_granted_date" in self.model_fields_set:
            _dict['atcIatcGrantedDate'] = None

        # set to None if atc_iatc_expiration_date (nullable) is None
        # and model_fields_set contains the field
        if self.atc_iatc_expiration_date is None and "atc_iatc_expiration_date" in self.model_fields_set:
            _dict['atcIatcExpirationDate'] = None

        # set to None if atc_iatc_pending_items (nullable) is None
        # and model_fields_set contains the field
        if self.atc_iatc_pending_items is None and "atc_iatc_pending_items" in self.model_fields_set:
            _dict['atcIatcPendingItems'] = None

        # set to None if pending_items_due_date (nullable) is None
        # and model_fields_set contains the field
        if self.pending_items_due_date is None and "pending_items_due_date" in self.model_fields_set:
            _dict['pendingItemsDueDate'] = None

        # set to None if dodinn_system_id (nullable) is None
        # and model_fields_set contains the field
        if self.dodinn_system_id is None and "dodinn_system_id" in self.model_fields_set:
            _dict['dodinnSystemId'] = None

        # set to None if authorization_to_use_connect_status (nullable) is None
        # and model_fields_set contains the field
        if self.authorization_to_use_connect_status is None and "authorization_to_use_connect_status" in self.model_fields_set:
            _dict['authorizationToUseConnectStatus'] = None

        # set to None if use_connect_authorization_date (nullable) is None
        # and model_fields_set contains the field
        if self.use_connect_authorization_date is None and "use_connect_authorization_date" in self.model_fields_set:
            _dict['useConnectAuthorizationDate'] = None

        # set to None if use_connect_authorization_termination_date (nullable) is None
        # and model_fields_set contains the field
        if self.use_connect_authorization_termination_date is None and "use_connect_authorization_termination_date" in self.model_fields_set:
            _dict['useConnectAuthorizationTerminationDate'] = None

        # set to None if terms_conditions_for_use_connect_summary (nullable) is None
        # and model_fields_set contains the field
        if self.terms_conditions_for_use_connect_summary is None and "terms_conditions_for_use_connect_summary" in self.model_fields_set:
            _dict['termsConditionsForUseConnectSummary'] = None

        # set to None if workstations (nullable) is None
        # and model_fields_set contains the field
        if self.workstations is None and "workstations" in self.model_fields_set:
            _dict['workstations'] = None

        # set to None if servers (nullable) is None
        # and model_fields_set contains the field
        if self.servers is None and "servers" in self.model_fields_set:
            _dict['servers'] = None

        # set to None if connectivity_ccsd (nullable) is None
        # and model_fields_set contains the field
        if self.connectivity_ccsd is None and "connectivity_ccsd" in self.model_fields_set:
            _dict['connectivityCcsd'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Systems from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "registrationCompletionDate": obj.get("registrationCompletionDate"),
            "systemLifeCycleAcquisitionPhase": obj.get("systemLifeCycleAcquisitionPhase"),
            "specialType": obj.get("specialType"),
            "specialTypeDescription": obj.get("specialTypeDescription"),
            "missionPortfolio": obj.get("missionPortfolio"),
            "isNNPI": obj.get("isNNPI"),
            "isRBC": obj.get("isRBC"),
            "isWaiver": obj.get("isWaiver"),
            "programOffice": obj.get("programOffice"),
            "vramId": obj.get("vramId"),
            "systemId": obj.get("systemId"),
            "policy": obj.get("policy"),
            "registrationType": obj.get("registrationType"),
            "name": obj.get("name"),
            "acronym": obj.get("acronym"),
            "description": obj.get("description"),
            "instance": obj.get("instance"),
            "owningOrganization": obj.get("owningOrganization"),
            "secondaryOrganization": obj.get("secondaryOrganization"),
            "versionReleaseNo": obj.get("versionReleaseNo"),
            "systemType": obj.get("systemType"),
            "isNSS": obj.get("isNSS"),
            "isPublicFacing": obj.get("isPublicFacing"),
            "coamsId": obj.get("coamsId"),
            "isTypeAuthorization": obj.get("isTypeAuthorization"),
            "ditprId": obj.get("ditprId"),
            "apmsId": obj.get("apmsId"),
            "vasiId": obj.get("vasiId"),
            "authorizationStatus": obj.get("authorizationStatus"),
            "authorizationDate": obj.get("authorizationDate"),
            "authorizationTerminationDate": obj.get("authorizationTerminationDate"),
            "authorizationLength": obj.get("authorizationLength"),
            "securityControlsAssessorExecutiveSummary": obj.get("securityControlsAssessorExecutiveSummary"),
            "riskReviewExecutiveSummary": obj.get("riskReviewExecutiveSummary"),
            "termsForAuth": obj.get("termsForAuth"),
            "securityPlanApprovalStatus": obj.get("securityPlanApprovalStatus"),
            "securityPlanApprovalDate": obj.get("securityPlanApprovalDate"),
            "missionCriticality": obj.get("missionCriticality"),
            "geographicalAssociation": obj.get("geographicalAssociation"),
            "systemOwnershipControlled": obj.get("systemOwnershipControlled"),
            "governingMissionArea": obj.get("governingMissionArea"),
            "primaryFunctionalArea": obj.get("primaryFunctionalArea"),
            "secondaryFunctionalArea": obj.get("secondaryFunctionalArea"),
            "primaryControlSet": obj.get("primaryControlSet"),
            "confidentiality": obj.get("confidentiality"),
            "integrity": obj.get("integrity"),
            "availability": obj.get("availability"),
            "appliedOverlays": obj.get("appliedOverlays"),
            "appliedStigs": obj.get("appliedStigs"),
            "rmfActivity": obj.get("rmfActivity"),
            "crossDomainTicket": obj.get("crossDomainTicket"),
            "ditprDonId": obj.get("ditprDonId"),
            "mac": obj.get("mac"),
            "dodConfidentiality": obj.get("dodConfidentiality"),
            "contingencyPlanRequired": obj.get("contingencyPlanRequired"),
            "contingencyPlanArtifact": obj.get("contingencyPlanArtifact"),
            "contingencyPlanTested": obj.get("contingencyPlanTested"),
            "contingencyPlanTestDate": obj.get("contingencyPlanTestDate"),
            "securityReviewRequired": obj.get("securityReviewRequired"),
            "securityReviewCompleted": obj.get("securityReviewCompleted"),
            "securityReviewCompletionDate": obj.get("securityReviewCompletionDate"),
            "nextSecurityReviewDueDate": obj.get("nextSecurityReviewDueDate"),
            "hasOpenPoamItem": obj.get("hasOpenPoamItem"),
            "hasOpenPoamItem90to120PastScheduledCompletionDate": obj.get("hasOpenPoamItem90to120PastScheduledCompletionDate"),
            "hasOpenPoamItem120PlusPastScheudledCompletionDate": obj.get("hasOpenPoamItem120PlusPastScheudledCompletionDate"),
            "impact": obj.get("impact"),
            "hasCUI": obj.get("hasCUI"),
            "hasPII": obj.get("hasPII"),
            "hasPHI": obj.get("hasPHI"),
            "ppsmRegistrationRequired": obj.get("ppsmRegistrationRequired"),
            "ppsmRegistryNumber": obj.get("ppsmRegistryNumber"),
            "ppsmRegistrationExemptionJustification": obj.get("ppsmRegistrationExemptionJustification"),
            "interconnectedInformationSystemAndIdentifiers": obj.get("interconnectedInformationSystemAndIdentifiers"),
            "privacyImpactAssessmentRequired": obj.get("privacyImpactAssessmentRequired"),
            "privacyImpactAssessmentStatus": obj.get("privacyImpactAssessmentStatus"),
            "privacyImpactAssessmentDate": obj.get("privacyImpactAssessmentDate"),
            "privacyImpactAssessmentArtifact": obj.get("privacyImpactAssessmentArtifact"),
            "userDefinedField1": obj.get("userDefinedField1"),
            "userDefinedField2": obj.get("userDefinedField2"),
            "userDefinedField3": obj.get("userDefinedField3"),
            "userDefinedField4": obj.get("userDefinedField4"),
            "userDefinedField5": obj.get("userDefinedField5"),
            "currentRmfLifecycleStep": obj.get("currentRmfLifecycleStep"),
            "otherInformation": obj.get("otherInformation"),
            "reportsForScorecard": obj.get("reportsForScorecard"),
            "highestSystemDataClassification": obj.get("highestSystemDataClassification"),
            "overallClassification": obj.get("overallClassification"),
            "isHVA": obj.get("isHVA"),
            "isFinancialManagement": obj.get("isFinancialManagement"),
            "isReciprocity": obj.get("isReciprocity"),
            "reciprocityExemption": obj.get("reciprocityExemption"),
            "cloudComputing": obj.get("cloudComputing"),
            "cloudType": obj.get("cloudType"),
            "authorizationToConnectStatus": obj.get("authorizationToConnectStatus"),
            "isSaaS": obj.get("isSaaS"),
            "isPaaS": obj.get("isPaaS"),
            "isIaaS": obj.get("isIaaS"),
            "otherServiceModels": obj.get("otherServiceModels"),
            "needDate": obj.get("needDate"),
            "overallRiskScore": obj.get("overallRiskScore"),
            "isHRR": obj.get("isHRR"),
            "connectivityAuthorizationDate": obj.get("connectivityAuthorizationDate"),
            "connectivityAuthorizationTerminationDate": obj.get("connectivityAuthorizationTerminationDate"),
            "systemDevelopmentLifeCycle": obj.get("systemDevelopmentLifeCycle"),
            "isFISMAReportable": obj.get("isFISMAReportable"),
            "groupTagging": obj.get("groupTagging"),
            "groupTagDescriptions": obj.get("groupTagDescriptions"),
            "dadmsId": obj.get("dadmsId"),
            "dadmsExpirationDate": obj.get("dadmsExpirationDate"),
            "enclaveConnectivity": obj.get("enclaveConnectivity"),
            "environmentType": obj.get("environmentType"),
            "navyCommonControlProvider": obj.get("navyCommonControlProvider"),
            "navyCloudBroker": obj.get("navyCloudBroker"),
            "cloudBrokerEmassId": obj.get("cloudBrokerEmassId"),
            "cloudBrokerProvisionalAuthorizationAtd": obj.get("cloudBrokerProvisionalAuthorizationAtd"),
            "navyJointAuthorization": obj.get("navyJointAuthorization"),
            "nmciNgenClins": obj.get("nmciNgenClins"),
            "enterpriseLocations": obj.get("enterpriseLocations"),
            "whitelistId": obj.get("whitelistId"),
            "whitelistInventory": obj.get("whitelistInventory"),
            "acquisitionCategory": obj.get("acquisitionCategory"),
            "softwareCategory": obj.get("softwareCategory"),
            "cybersecurityServiceProvider": obj.get("cybersecurityServiceProvider"),
            "cybersecurityServiceProviderExceptionJustification": obj.get("cybersecurityServiceProviderExceptionJustification"),
            "maximumTolerableDowntime": obj.get("maximumTolerableDowntime"),
            "recoveryTimeObjective": obj.get("recoveryTimeObjective"),
            "recoveryPointObjective": obj.get("recoveryPointObjective"),
            "businessImpactAnalysisRequired": obj.get("businessImpactAnalysisRequired"),
            "businessImpactAnalysisArtifact": obj.get("businessImpactAnalysisArtifact"),
            "incidentResponsePlanRequired": obj.get("incidentResponsePlanRequired"),
            "incidentResponsePlanArtifact": obj.get("incidentResponsePlanArtifact"),
            "disasterRecoveryPlanRequired": obj.get("disasterRecoveryPlanRequired"),
            "disasterRecoveryPlanArtifact": obj.get("disasterRecoveryPlanArtifact"),
            "privacyThresholdAnalysisCompleted": obj.get("privacyThresholdAnalysisCompleted"),
            "privacyThresholdAnalysisDate": obj.get("privacyThresholdAnalysisDate"),
            "privacyThresholdAnalysisArtifact": obj.get("privacyThresholdAnalysisArtifact"),
            "privacyActSystemOfRecordsNoticeRequired": obj.get("privacyActSystemOfRecordsNoticeRequired"),
            "eAuthenticationRiskAssessmentRequired": obj.get("eAuthenticationRiskAssessmentRequired"),
            "eAuthenticationRiskAssessmentDate": obj.get("eAuthenticationRiskAssessmentDate"),
            "eAuthenticationRiskAssessmentArtifact": obj.get("eAuthenticationRiskAssessmentArtifact"),
            "ipv4OnlyAssets": obj.get("ipv4OnlyAssets"),
            "ipv6OnlyAssets": obj.get("ipv6OnlyAssets"),
            "ipv4Ipv6DualStackAssets": obj.get("ipv4Ipv6DualStackAssets"),
            "totalIpAssets": obj.get("totalIpAssets"),
            "originatingOrganization": obj.get("originatingOrganization"),
            "systemUseJustification": obj.get("systemUseJustification"),
            "systemUseJustificationArtifact": obj.get("systemUseJustificationArtifact"),
            "authorizationToUseStatus": obj.get("authorizationToUseStatus"),
            "reciprocityAcceptanceStatus": obj.get("reciprocityAcceptanceStatus"),
            "useAuthorizationDate": obj.get("useAuthorizationDate"),
            "reciprocityAcceptanceDate": obj.get("reciprocityAcceptanceDate"),
            "useAuthorizationTerminationDate": obj.get("useAuthorizationTerminationDate"),
            "reciprocityAcceptanceTerminationDate": obj.get("reciprocityAcceptanceTerminationDate"),
            "termsConditionsForUseSummary": obj.get("termsConditionsForUseSummary"),
            "termsConditionsForReciprocitySummary": obj.get("termsConditionsForReciprocitySummary"),
            "primaryMissionEssentialFunction": obj.get("primaryMissionEssentialFunction"),
            "pmefDescription": obj.get("pmefDescription"),
            "missionEssentialFunction": obj.get("missionEssentialFunction"),
            "mefDescription": obj.get("mefDescription"),
            "administration": obj.get("administration"),
            "administrationOtherJustification": obj.get("administrationOtherJustification"),
            "atcIatcGrantedDate": obj.get("atcIatcGrantedDate"),
            "atcIatcExpirationDate": obj.get("atcIatcExpirationDate"),
            "atcIatcPendingItems": obj.get("atcIatcPendingItems"),
            "pendingItemsDueDate": obj.get("pendingItemsDueDate"),
            "dodinnSystemId": obj.get("dodinnSystemId"),
            "authorizationToUseConnectStatus": obj.get("authorizationToUseConnectStatus"),
            "useConnectAuthorizationDate": obj.get("useConnectAuthorizationDate"),
            "useConnectAuthorizationTerminationDate": obj.get("useConnectAuthorizationTerminationDate"),
            "termsConditionsForUseConnectSummary": obj.get("termsConditionsForUseConnectSummary"),
            "workstations": obj.get("workstations"),
            "servers": obj.get("servers"),
            "connectivityCcsd": [ConnectivityCcsd.from_dict(_item) for _item in obj["connectivityCcsd"]] if obj.get("connectivityCcsd") is not None else None
        })
        return _obj


